/*
Copyright (C) 2006-2008 Keio University
(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)

This file is part of Murasaki.

Murasaki is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Murasaki is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Murasaki.  If not, see <http://www.gnu.org/licenses/>.
*/

//////////////
// murasaki project
// sequence.cc
// provides "sequence" class for doing operations on dna sequences
//////////////

#include "murasaki.h"
#include "sequence.h"
#include "dinkymath.h"
#include "cmultiset.h"
#include "ecohash.h"
#include "murasaki_mpi.h"
#include "seqread.h"
#include "mingw32compat.h"

#include <fstream>
#include <limits.h>
#include <assert.h>
#include <set>
#include <iterator>
#include <functional>
#include <algorithm>
#include <stdlib.h> //need random()
#include <math.h> //need log...
#include <ctype.h> //for tolower <-- im a lazy lazy man
//#include <error.h>
#include <errno.h>
#include <unistd.h>

//ahh boosty my old friend...
#include <boost/spirit/core.hpp>
#include <boost/spirit/attribute.hpp>
#include <boost/spirit/symbols/symbols.hpp>
#include <boost/spirit/utility/chset.hpp>
#include <boost/spirit/utility/escape_char.hpp>
#include <boost/spirit/utility/confix.hpp>
#include <boost/spirit/iterator.hpp>
#include <boost/spirit/error_handling/exceptions.hpp>

#ifdef USE_SHM_MMAP
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>

#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
namespace fs = boost::filesystem;
#endif

#ifdef USE_SHM_SYSV
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif

//these guys are used in creating the adaptive hash stuff:
#include <ext/functional>
using namespace __gnu_cxx;

using namespace std;


int next_sequence_=0;
int _wordsize=WORDSIZE;
int hash_bits=26; 
int max_hash_bits=26; //keeps the approx 256mb hash as default (or 515mb on 64bit)
word hash_mask=lowN(hash_bits);
int hash_max=intpow(2,max_hash_bits);
//int hash_max=67108879; //first prime over .5gb/8bytes
//524288057; //arbitrary default. first prime number past .5gb
word hash_size;

word alternating_mask;
word activeHash; //for building hash member lists
word totalSequenceMemory=0;
word sys_pagesize=getpagesize();

//static inits
int HashMethod::nextId=0;
map<int,int> HashMethod::popmap;

//seqtable bits
Sequence** seq_table;
SeqIdx seq_table_size;
map<Sequence*,SeqIdx> seq2idx;
bool _seq_no_data=false;
double globalBaseFreq[4]={0,0,0,0};
word globalBaseCount[4]={0,0,0,0};
word globalCounted=0;

void set_seqtable(vector<Sequence*> seqs){
  seq_table_size = seqs.size();

  seq_table = (Sequence**)malloc(sizeof(Sequence*)*seq_table_size);
  if(seq_table == NULL) { cerr << "set_seqtable(): malloc() failed." << endl; exit(-1); }
  for(SeqIdx i=0; i<seq_table_size; i++){
    seq_table[i] = seqs[i];
    seq2idx[seqs[i]] = i;
  }  
}

void initHashParams(int bits,BitSequence *pat){
  hash_bits=bits;
  hash_mask=lowN(bits);
  hash_size=intpow(2,bits);
  cout << "Using "<<bits<<" hashbits"<<endl;
  srandom(opt.rand_seed);
#ifndef NDEBUG
  srandom(0);
#endif
  if(opt.quick_hash==3)
    pat->compileHashFunc();
  if(opt.hashPref==EcoHash_t)
    EcoHash::init(seq_count,longestSeq,totalHashLength,bits);
}

void initConstants(){
  srandom(time(0));
  alternating_mask=1;
  //make a mask like 010101010101010101
  for(int bit=2;bit<WORDSIZE;bit<<=1)
    alternating_mask|= (alternating_mask<<bit);
}

inline word revComp8bit(word t){
  switch(t & 255){
#include "revcomptable.cxx"    
  }
  throw MurasakiDebugException("revcomp8bit switch table is broken.");
}

inline word revCompWord(word t){
  word res=0;
  for(int i=0;i<WORDSIZE;i+=8)
    res=(res<<8) | revComp8bit(t>>i);
  return res;
}

int highestBit(word t){
  for(int i=WORDSIZE-4;i>=0;i-=4){
    switch((t>>i) & (word)15){
    case 1:return i;
    case 2:case 3:return i+1;
    case 4:case 5:case 6:case 7:return i+2;
    case 8:case 9:case 10:case 11:case 12:case 13:case 14:case 15:
      return i+3;
      //this could be unrolled arbitrarily for performance
    }
  }
  return -1;//wtf?
}

int lowestBit(word t){
  for(int i=0;i<WORDSIZE;i+=4){
    switch((t>>i) & (word)15){
    case 1:case 3:case 5:case 7: case 9: case 11: case 13: case 15:
      return i;
    case 2:case 6:case 10:case 14:return i+1;
    case 4:case 12:return i+2;
    case 8:return i+3;
      //this could be unrolled arbitrarily for performance
    }
  }
  return -1;//wtf?
}

int matchCount(word t){ //count whole-base matches
  int count=0;
  for(int i=0;i<WORDSIZE;i+=4){
    switch((t>>i) & (word)15){ //4 bits at a time
    case 3:case 12:count+=1;break;
    case 15:count+=2;break;
    default:break;
    }
  }
  return count;
}

int popCount(word t){
  int count=0;
  for(int i=0;i<WORDSIZE;i+=4){
    switch((t>>i) & (word)15){
    case 1:case 2:case 4:case 8:count+=1;break;
    case 3:case 5:case 6:case 9:case 10:case 12:count+=2;break;
    case 7:case 11:case 13:case 14:count+=3;break;
    case 15:count+=4;
      //this could be unrolled arbitrarily for performance
    }
  }
  return count;
}

SeqPos Sequence::length() const{
  return fwd->length();
}

Sequence::Sequence()
  : 
  fwd(),rev(),seqID(next_sequence_++)
#ifdef USE_SHM_SYSV
  ,sysv_shmid(-1)
#endif
{
}

Sequence::Sequence(string _filename):
  filename(_filename),name(_filename),
  seqID(next_sequence_++)
#ifdef USE_SHM_SYSV
  ,sysv_shmid(-1)
#endif
{
#ifdef MURASAKI_MPI
  if(mpi_capable){
    word length;
    if(opt.mpi_fileDistro){
      if(mpi_id==0){
	cout << "Parsing sequence data from "<<filename<<endl;
	try {
	  SequenceFileReader reader(filename,opt.seqreadOptions);
	  baseFilename=reader.get_filename(); //might be a subsequence of something
	  fwd=new BitSequence(reader,this);
	}catch(SeqReadException e){
	  throw MurasakiException("Error reading "+filename+": "+e.reason());
	}
	length=(word)fwd->length();
	if(!length){
	  throw MurasakiException("Can't run Murasaki with empty sequences");
	}
	MPI_Bcast(&length,1,MPI_UNSIGNED_LONG,0,MPI_COMM_WORLD);
	cout << "Distributing "<<filename<<" via MPI"<<endl;
      }else{
	baseFilename=filename;
	pair<size_t,size_t> range; //rangeSpec wants to sue this
	SequenceFileReader::parseRangeSpec(baseFilename,range,opt.seqreadOptions);
	cout << "Waiting for node 0 to finish parsing "<<filename<<endl;
	MPI_Bcast(&length,1,MPI_UNSIGNED_LONG,0,MPI_COMM_WORLD);
	fwd=new BitSequence(length,this);
	cout << "Receiving "<<filename<<" from node 0"<<endl;
      }
    }else{
	cout << "Parsing sequence data from "<<filename<<endl;
	try {
	  SequenceFileReader reader(filename,opt.seqreadOptions);
	  baseFilename=reader.get_filename(); //might be a subsequence of something
	  fwd=new BitSequence(reader,this);
	}catch(SeqReadException e){
	  throw MurasakiException("Error reading "+filename+": "+e.reason());
	}
	if(!fwd->length()){
	  throw MurasakiException("Can't run Murasaki with empty sequences");
	}
    }
    //synch str with other procs
    fwd->mpi_distribute();

    cout << "Done with forward strand ("<<fwd->length()<<"bp). Generating reverse complement..."<<endl;
    rev=fwd->reverseComplement();
  
    if(mpi_usingShm){
      if(!mpi_isHostLeader)
	cout << "Waiting for leader nodes to finish preparing sequence."<<endl;
      MPI_Barrier(MPI_COMM_WORLD);
      if(mpi_isHostLeader)
	cout << "Distributing to listener nodes."<<endl;
      else
	cout << (opt.use_shm_mmap ? "Mapping in shared pages from leader node.":"Finalizing shared region data.")<<endl;
      fwd->shm_distribute();
      rev->shm_distribute();
    }
  }else{
#endif
    cout << "Parsing sequence data from "<<filename<<endl;
    try {
      SequenceFileReader reader(filename,opt.seqreadOptions);
      fwd=new BitSequence(reader,this);
    }catch(SeqReadException e){
      throw MurasakiException("Error reading "+filename+": "+e.reason());
    }
    if(!fwd->length()){
      throw MurasakiException("Can't run Murasaki with empty sequences");
    }
    cout << "Done with forward strand ("<<fwd->length()<<"bp). Generating reverse complement..."<<endl;
    rev=fwd->reverseComplement();
#ifdef MURASAKI_MPI
  }
#endif
  cout << "Done."<<endl;
  if(length()<100){
    cout << "Fwd contents:\n"<<*fwd<<endl;
    cout << "Rev contents:\n"<<*rev<<endl;
  }
}

Sequence::Sequence(const Sequence &a):
  filename(a.filename),
  name(a.name),
  fwd(a.fwd),
  rev(a.rev),  seqID(a.seqID)
#ifdef USE_SHM_SYSV
  ,sysv_shmid(a.sysv_shmid)
#endif
{
  assert(cerr << "Alert: Copying sequence: "<<a.name<<endl);
}

Sequence::~Sequence(){
  assert(cerr << "Alert: Destructing "<<name<<endl);
#ifdef USE_SHM_SYSV
  if(opt.use_shm_sysv){
    if(sysv_shmid==-1){//never been used?
      assert(cerr << "Alert: "<<name<<" never got assigned a shared memory region?"<<endl);
    }else{
      if(shmctl(sysv_shmid,IPC_RMID,NULL)<0)
	throw MurasakiException("Couldn't mark System V IPC shared memory region for deletion for sequence "+name+strerror(errno));
    }
  }
#endif
  delete fwd;
  delete rev;
}

//inline int Sequence::getId(){return seqID;}

//inline const vector<word>& BitSequence::words(){return _words;}
int BitSequence::wordsize(){return WORDSIZE;};

inline int BitSequence::readCode(SeqPos pos) const {
  word w=readWord(pos/WORDSIZE);
  return first2(w<<(MODWORDSIZE(pos)));
}

inline word BitSequence::readWord(SeqPos pos) const{
  if(pos>=(SeqPos)word_count || pos<0)
    return 0;
  return (opt.reverseOtf && reverse) ? reverseOtf(pos):_words[pos];
}

inline word BitSequence::reverseOtf(SeqPos pos) const{
  int bitOffset=MODWORDSIZE(bit_count);
  if(bitOffset){
    if((word)pos<word_count-1){ //if it's not the very last word, it's actually composed of 2 words' data...
      return (revCompWord(_words[word_count-pos-1])<<(WORDSIZE-bitOffset)) |
	(revCompWord(_words[word_count-pos-2])>>(bitOffset));
    }
    else
      return (revCompWord(_words[word_count-pos-1])<<(WORDSIZE-bitOffset)); //last word is a lonely muffin.
  }
  return revCompWord(_words[word_count-pos-1]);
}

void BitSequence::shm_distribute(){
 //no distribution in non-mpi modes
#ifndef MURASAKI_MPI
  throw MurasakiDebugException("Distribute called without MPI support compiled??");
  return;
#else
  if(!mpi_capable)
    throw MurasakiDebugException("Distribute called without MPI enabled??");

  if(!opt.use_shm_sysv && !opt.use_shm_mmap)
    throw MurasakiDebugException("Distribute called without any shared memory modes enabled.");

#ifdef USE_SHM_SYSV
  if(opt.use_shm_sysv){
    if(reverse || !seq)
      return; //no distribution necessary. w00t.

    //on the forward strand
    //however now that we know that everyone has their sequence mapped and attached,
    //if we do the rm on the segment now, we'll prevent memory leaks in a segfaulty disaster
    if(seq->sysv_shmid==-1){//never been used?
      assert(cerr << "Alert: "<<seq->name<<" never got assigned a shared memory region?"<<endl);
    }else{
      if(opt.verbose)cout << "Tagging seq "<<seq->name<<"'s sysv region for deletion."<<endl;
      if(shmctl(seq->sysv_shmid,IPC_RMID,NULL)<0)
	throw MurasakiException("Couldn't mark System V IPC shared memory region for deletion for sequence "+seq->name+strerror(errno));
    }
    return;
  }
#endif
#ifdef USE_SHM_MMAP
  if(opt.use_shm_mmap){
    int fd;
    void *mem;
    const int NFS_Timeout=60;
    fs::path bitFile;
    switch(shmUsed){
    case SHM_MMAP_RW: //nothing, we're done.
      break;
    case SHM_MMAP_RO: //open file and mmap it.
      if(opt.reverseOtf && reverse){
	assert(seq);assert(seq->fwd);assert(seq->fwd->_words);
	_words=seq->fwd->_words; //attach to fwd seq's words, and we're done
	break;
      }
      bitFile=fs::path(genBitFilename());
      for(int trial=0;!fs::exists(bitFile);trial++){
	if(trial+1>NFS_Timeout){
	  throw MurasakiException("Gave up waiting for "+bitFile.string()+" to appear in filesystem.");
	}
	cerr << bitFile.string() << " not found. Waiting for filesystem to catch up..."<<endl;
	sleep(1);
      }
      if((fd=open(bitFile.string().c_str(),O_RDONLY))<0){
	throw MurasakiException("Couldn't open 2bit storage file for reading: "+bitFile.string()+" : "+strerror(errno));
      }
      if((mem=mmap(NULL,word_count*sizeof(word),PROT_READ,MAP_SHARED | MAP_POPULATE,fd,0))==MAP_FAILED){
	throw MurasakiException("Failed mapping listener block: "+string(strerror(errno)));
      }
      _words=(word*)mem;
      break;
    default:
      assert(cerr << "Don't have a shm mode assigned??"<<endl &&  0);
    }

    return;
  }
#endif
  throw MurasakiDebugException("BitSequence::shm_distribute called without shared memory support compiled");
  return;
#endif
}

void BitSequence::mpi_distribute(){
#ifdef MURASAKI_MPI
  if(mpi_capable){
    assert(_words);
    if(opt.mpi_fileDistro){
      word *start=_words;
      word sent=0;
      if(mpi_isHostLeader){ //leaders only!
	while(sent<word_count){
	  word block=min<word>(word_count-sent,INT_MAX>>1); //silly MPI specifies transmission count in ints...
	  cout << (mpi_id==0 ? "Sending a ":"Receiving a ")<<humanMemory(block*sizeof(word))<<" block."<<endl;
	  MPI_Bcast(start+sent,block,MPI_UNSIGNED_LONG,0,mpi_leaders_comm);
	  sent+=block;
	  cout << percent((double)sent,(double)word_count) << (mpi_id==0 ? " sent":" received")<<endl;
	}
      }else
	cout << "My leader's receiving a copy of the sequence for me."<<endl;
    }

    //and distribute meta-data!
    if(opt.mpi_fileDistro || mpi_usingShm){
      cout << "Synchronizing sequence meta-data:"<<endl;
      word regionCount=matchRegions.size();
      MPI_Bcast(&regionCount,1,MPI_UNSIGNED_LONG,0,opt.mpi_fileDistro ? MPI_COMM_WORLD:mpi_localhost_comm);
      cout << " : "<<regionCount<< (regionCount==1 ? " region":" regions")<< " ("<<humanMemory(sizeof(pair<SeqPos,SeqPos>)*regionCount)<<")"<<endl;
      if(opt.mpi_fileDistro ? mpi_id!=0:!mpi_isHostLeader)
	matchRegions.resize(regionCount);
      MPI_Bcast(&matchRegions.front(),sizeof(pair<SeqPos,SeqPos>)*matchRegions.size(),MPI_BYTE,0,opt.mpi_fileDistro ? MPI_COMM_WORLD:mpi_localhost_comm);
    }
  }else
#endif
    throw MurasakiDebugException("BitSequence::mpi_distribute called without MPI support!");
}

string BitSequence::genBitFilename(){
  assert(seq);
  assert(!seq->filename.empty());
  return seq->filename+string(reverse ? ".2bit.rev":".2bit.fwd")+string(opt.repeatMask ? ".repeatmasked":"");
}

string bitToString(word w){
  switch(w & (word)3){
  case 0:return "C"; break;
  case 1:return "A"; break;
  case 2:return "T"; break;
  case 3:return "G"; break;
  }
  return " ";
}

string wordToString(word w,int bits){
  string ret;
  int start=bits < 0 ? bits+WORDSIZE:0;
  for(int i=start;i<WORDSIZE;i+=2)
    switch(first2(w<<i)){
    case 0:ret+="C"; break;
    case 1:ret+="A"; break;
    case 2:ret+="T"; break;
    case 3:ret+="G"; break;
    }
  return ret;
}

string wordToPattern(word w,int bits){
  string ret;
  for(int i=0;i<bits;i+=2)
    switch(first2(w<<i)){
    case 0:ret+="0"; break;
    case 1:ret+="-"; break;
    case 2:ret+="x"; break;
    case 3:ret+="1"; break;
    }
  return ret;
}

string BitSequence::asString() const {
  string ret;
  const int wordsPerLine=(80/WORDSIZE*2);
  for(word i=0;i<word_count;i++){
    if(i%wordsPerLine==0){
      char buf[50];
      sprintf(buf,"%4lu ",i*WORDSIZE/2+1);
      ret+=buf;
    }
    string seg=wordToString(readWord(i));
    if(i>=word_count-1)
      seg.resize((((bit_count-1)%WORDSIZE)+1)/2);
    ret+=seg;
      
    if(i<word_count-1 && (i+1)%wordsPerLine==0)
	ret+="\n";
      else
	ret+=" ";
  }
  return ret;
}

string BitSequence::asPattern() const {
  string ret;
  for(word i=0;i<word_count;i++){
    if(i%(80/WORDSIZE*2)==0){ //used to be i%4 but that looks weird on hpc
      char buf[50];
      sprintf(buf,"%4lu ",i*WORDSIZE/2+1);
      ret+=buf;
    }
    ret+=wordToPattern(readWord(i),i<word_count-1 ? WORDSIZE:
		      ((bit_count-1)%WORDSIZE)+1);
    if(i<word_count-1 && (i+1)%(80/WORDSIZE*2)==0)
	ret+="\n";
      else
	ret+=" ";
  }
  return ret;
}

int BitSequence::compHashLength(){
  int count=0;
  for(word i=0;i<word_count;i++){
    count+=popCount(readWord(i))/2;
  }
  _hashLength=count;
  return count;
}

//operator
SeqPos BitSequence::cmpRight(SeqPos base_pos,BitSequence& target,SeqPos base_targetPos){
  if(base_pos>=_length || base_targetPos>=target._length)return 0; //no can do, mack
  SeqPos pos=2*base_pos;//to bitspace
  SeqPos targetPos=2*base_targetPos;

  SeqPos myOffset=MODWORDSIZE(pos),tOffset=MODWORDSIZE(targetPos);
  word myFrame=pos/WORDSIZE,tFrame=targetPos/WORDSIZE;
  word myWord=readWord(myFrame),myNext=readWord(myFrame+1);
  word tWord=target.readWord(tFrame),tNext=target.readWord(tFrame+1);

  SeqPos max=min<SeqPos>(spaceRight(base_pos),target.spaceRight(base_targetPos));
  //  assert(cout << "local Spaceright: "<<spaceRight(base_pos)<< " target spaceright: "<< target.spaceRight(base_targetPos)<<endl);
  //  assert(cout << "Valid range: "<<(base_pos)<<","<<(base_pos+max)<<endl);
  if(!max)return 0; //if dead, die
  SeqPos matched=0;
  while(myFrame<word_count && tFrame<target.word_count){
    if(myOffset)
      myWord=(myWord<<myOffset) | ((myNext>>(WORDSIZE-myOffset)));
    if(tOffset)
      tWord=(tWord<<tOffset) | ((tNext>>(WORDSIZE-tOffset)));
    word res=(tWord ^ myWord);
    //    assert(cout << "Cmp right -> "<<wordToString(myWord)<<" to "<<wordToString(tWord) << " = "
    //	   << wordToString(res) << endl);
    if(res){ //sucky, not a whole match
      SeqPos highest=highestBit(res);
      matched+=WORDSIZE/2-1-(highest/2);
      break; //game over man, game over!
    }
    else
      matched+=WORDSIZE/2;
    
    myFrame++;tFrame++;
    myWord=myNext;myNext=readWord(myFrame+1);
    tWord=tNext;tNext=target.readWord(tFrame+1);
  }
  assert(matched>=0);
  if(matched>=max)
    matched=max;
  assert(matched>=0);
  return matched;
}

SeqPos BitSequence::cmpLeft(SeqPos base_pos,BitSequence& target,SeqPos base_targetPos){
  if(base_pos < 0 || base_targetPos < 0)return 0; //no can do, mack
  SeqPos pos=2*(base_pos);//to bitspace
  SeqPos targetPos=2*(base_targetPos);
  SeqPos myOffset=MODWORDSIZE(pos+2),tOffset=MODWORDSIZE(targetPos+2);
  SeqPos myFrame=pos/WORDSIZE,tFrame=targetPos/WORDSIZE;
  word myWord=readWord(myFrame),myNext=readWord(myFrame-1);
  word tWord=target.readWord(tFrame),tNext=target.readWord(tFrame-1);

  SeqPos max=min<SeqPos>(spaceLeft(base_pos),target.spaceLeft(base_targetPos));
  if(!max)return 0; //if dead, die
  SeqPos matched=0;
  while(myFrame>=0 && tFrame>=0){
    //    cout << "myOffset: "<<myOffset<<" tOffset: "<<tOffset<<endl;
    //    cout << "*Slice: "<<wordToString(myWord>>(WORDSIZE-myOffset)) << " | " << wordToString(myNext<<myOffset) << endl;
    //    cout << "& Dice: "<<wordToString(tWord) << " | "<< wordToString(tNext)<<endl;
    if(myOffset)
      myWord=(myWord>>(WORDSIZE-myOffset)) | ((myNext<<(myOffset)));
    if(tOffset)
      tWord=(tWord>>(WORDSIZE-tOffset)) | ((tNext<<(tOffset)));
    word res=(tWord ^ myWord);
    //    assert(cout << "Cmp Left <- "<<wordToString(myWord)<<" to "<<wordToString(tWord) << " = "
    //    	   << wordToString(res) << endl);
    if(res){ //sucky, not a whole match
      SeqPos highest=lowestBit(res);
      matched+=(highest/2);
      break; //game over man, game over!
    }
    else
      matched+=WORDSIZE/2;
    
    myFrame--;tFrame--;
    myWord=myNext;myNext=readWord(myFrame-1);
    tWord=tNext;tNext=target.readWord(tFrame-1);
  }
  assert(matched>=0);
  if(matched>=max)
    matched=max;
  assert(matched>=0);
  return matched;
}

bool BitSequence::equal(SeqPos pos,BitSequence& target,SeqPos targetPos,BitSequence& patt){
  pos*=2;//to bitspace
  targetPos*=2; 
  SeqPos myOffset=MODWORDSIZE(pos),tOffset=MODWORDSIZE(targetPos);
  word myMask=highN(myOffset),tMask=highN(tOffset);
  word myFrame=pos/WORDSIZE,tFrame=targetPos/WORDSIZE;
  word myWord=readWord(myFrame),myNext=readWord(myFrame+1);
  word tWord=target.readWord(tFrame),tNext=readWord(tFrame+1);

  word pFrame=0;
  word pWord;

  SeqPos matched=0;
  while(myFrame<word_count && tFrame<target.word_count && pFrame<patt.word_count){
    pWord=patt.readWord(pFrame%patt.word_count); //pattern just loops for now
    if(myOffset)
      myWord=(myWord<<myOffset) | (myNext<<myOffset & ~myMask);
    if(tOffset)
      tWord=(tWord<<tOffset) | (tNext<<tOffset & ~tMask);
    word res=(tWord ^ myWord) & pWord;
    if(res){ //sucky, not a whole match
      SeqPos highest=highestBit(tWord);
      if(highest>=0) //partial match
	matched+=highest/2+1;
      break; //game over man, game over!
    }
    else
      matched+=WORDSIZE/2;
    
    myFrame++;tFrame++;pFrame++;
    myWord=myNext;myNext=readWord(myFrame+1);
    tWord=tNext;tNext=readWord(tFrame+1);
  }
  return matched>=patt._length;
}

pair<SeqPos,SeqPos>* Location::localRegion(){ 
  //  cout << "Converting pos "<<pos<<" on "<<seqno<<" into a region.\n";
  return (pos>0 ? seq()->fwd->localRegion(pos-1):seq()->rev->localRegion(seq()->length()+pos));
}

vector<pair<SeqPos,SeqPos> >::iterator Location::localRegionIte(){ //find region local to pos
  //  cout << "Converting "<<(*this)<<" into a local region. pos is "<<pos;
  SeqPos bitseqp=(pos>0 ? pos:0-pos)-1;
  //  cout << ", but bitseqp is "<<bitseqp<<endl;
  vector<pair<SeqPos,SeqPos> > &regions=bitSeq()->matchRegions;
  pair<SeqPos,SeqPos> target(bitseqp,bitseqp); //for simplicity
  vector<pair<SeqPos,SeqPos> >::iterator
    result=lower_bound(regions.begin(),regions.end(),target,ltRegion());
  if(result==regions.end() || bitseqp<result->first){
    if(result==regions.end())assert(cout << "bitseqp "<<bitseqp << " not in any region"<<endl && 0);
    if(bitseqp<result->first)assert(cout << "bitseqp "<<bitseqp<<" falls before start of next region"<<endl && 0);
    return regions.end();
  }
  //  assert(cout << "region found: "<<result->first<<","<<result->second<<endl);
  return result;
}

pair<SeqPos,SeqPos>* BitSequence::localRegion(SeqPos pos){ //find region local to pos
  pair<SeqPos,SeqPos> target(pos,pos); //for simplicity
  vector<pair<SeqPos,SeqPos> >::iterator
    result=lower_bound(matchRegions.begin(),matchRegions.end(),target,ltRegion());
  if(result==matchRegions.end()){
    //    assert(cout << "pos "<<pos << " not in any region"<<endl);
    return 0;
  }
  if(pos<result->first){
    //    assert(cout << "pos "<<pos<<" falls before start of next region"<<endl);
    return 0;
  }
  //  assert(cout << "region found: "<<result->first<<","<<result->second<<endl);
  return &(*result);
}

SeqPos BitSequence::spaceRight(SeqPos pos){ //find region local to pos
  pair<SeqPos,SeqPos> target(pos,pos); //for simplicity
  pair<SeqPos,SeqPos> *region(localRegion(pos));
  if(!region){
    return 0;
  }
  return region->second-pos+1;
}

SeqPos BitSequence::spaceLeft(SeqPos pos){ //find region local to pos
  pair<SeqPos,SeqPos> target(pos,pos); //for simplicity
  pair<SeqPos,SeqPos> *region(localRegion(pos));
  if(!region){
    return 0;
  }
  return pos-region->first+1;
}

//mutator
void BitSequence::invert(){ //flips to reverse comp sequence
  for(word i=0;i<word_count;i++){
    _words[i]= ~_words[i];
  }
  //last word is tricky. all trailing elements have to be C in order
  //to not screw up dumb hashing techniques
  SeqPos last=word_count-1;
  _words[last]=_words[last] & highN(MODWORDSIZE(bit_count));
}

word* BitSequence::allocWords(){
  word *mem;
  assert(!_words); //better not already have words allocated....!
  if(opt.verbose)cout << "Allocing words...";

  if(reverse && opt.reverseOtf){
    assert(seq);assert(seq->fwd);assert(seq->fwd->_words);
    if(opt.verbose)cout << "as mirror of fwd"<<endl;
    return seq->fwd->_words; //shares the same data, and we're done.
  }

#ifdef MURASAKI_MPI
#ifdef USE_SHM_SYSV
  if(opt.use_shm_sysv && seq){
    if(opt.verbose)cout << "using sysv";
    size_t size=word_count*sizeof(word);
    if(!opt.reverseOtf)
      size*=2;//fetch double the size because we're storing both fwd and rev in there
    //sysv segments are ephemereal and local to this mpi set, so we just use the filename as our key
    shmUsed=SHM_SYSV;
    if(reverse){
      assert(seq->fwd->_words); //MUST allocate fwd before rev.
      if(opt.verbose)cout << " (continued from fwd allocation)"<<endl;
      return seq->fwd->_words+word_count; //second half of block
    }
    if(opt.verbose)cout << " (new segment)"<<endl;
    //ok, going to create a new segment
    if((seq->sysv_key=ftok(seq->baseFilename.c_str(),mpi_sysv_projid))==-1)
      throw MurasakiException("Error generating System V IPC key for "+string(seq->filename)+": "+strerror(errno));
    if((seq->sysv_shmid=shmget(seq->sysv_key,size,IPC_CREAT | 00600))==-1)
      throw MurasakiException("Error creating System V IPC shared memory segment (size: "+humanMemory(size)+") for "+string(seq->filename)+": "+strerror(errno));
    if((mem=(word*)shmat(seq->sysv_shmid,NULL,mpi_isHostLeader ? O_RDWR:(O_RDONLY | SHM_RDONLY)))==(void*)-1)
      throw MurasakiException("Error attaching to System V IPC shared memory segment for "+string(seq->filename)+strerror(errno));
    cout << "Mapped a "<<humanMemory(size)<<" sysv chunk (key: "<<dhstring(seq->sysv_key)<<" shmid: "<<seq->sysv_shmid<<")"<<endl;

    if(mpi_isHostLeader) //keep track of how much mem is lost to sequence storage
      totalSequenceMemory+=((((size-1)/sys_pagesize)+1)*sys_pagesize)/1024;
    return mem; //wee. done. much cleaner than mmap.
  }
#endif
#ifdef USE_SHM_MMAP
  if(opt.use_shm_mmap && seq){
    if(opt.verbose)cout << "using mmap"<<endl;
    if(opt.reverseOtf && reverse){
      assert(seq);assert(seq->fwd);
      shmUsed=SHM_MMAP_RO; //we'll pretend to be RO.
      return 0; 
    }
    //prepare to write my bitsequence to file if needed
    fs::path bitFile(genBitFilename());
    bool freshen=(!fs::exists(bitFile) || fs::last_write_time(bitFile)<fs::last_write_time(seq->filename));
    int fd; //actually we never need this again...
    if(freshen && ((opt.mmap_writePerHost && mpi_hostLeader[mpi_id]==mpi_id) ||
		   (mpi_id==0))){
      cerr << "Writing 2bit data out to file..."<<endl;
      fd=open(bitFile.string().c_str(),O_RDWR | O_CREAT | O_TRUNC | O_NOCTTY,S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH); //create and/or trunc
      if(fd<0)
	throw MurasakiException("Couldn't create 2bit storage file: "+bitFile.string()+" : "+strerror(errno));
      //make file big enough to fit our sequence
      lseek(fd,word_count*sizeof(word)-1,SEEK_SET);
      write(fd,"",1); //write a byte.
      mem=(word*)mmap(NULL,word_count*sizeof(word),PROT_READ|PROT_WRITE,MAP_SHARED,fd,0);
      if(mem==MAP_FAILED){
	int mmap_error=errno;
	throw MurasakiException("Failed mapping initial block: "+string(strerror(mmap_error)));
      }
      totalSequenceMemory+=((word_count*sizeof(word))/(sys_pagesize-1)*sys_pagesize/1024);
      shmUsed=SHM_MMAP_RW;
    }else{
      //don't write just load from file when distribute is called.
      mem=0;
      shmUsed=SHM_MMAP_RO;
    }
    return mem;
  }
#endif
#endif
  mem=new word[word_count];
  totalSequenceMemory+=(word_count*sizeof(word));

  if(opt.verbose)cout << "directly"<<endl;
  return mem;
}

//init
void BitSequence::initPattern(const string &str){
  _hashLength=0;
  _length=str.length();
  bit_count=_length*2;
  word_count=((bit_count+WORDSIZE-1)/(WORDSIZE)); //rounds up to nearest full word
  assert(_words==0);
  _words=allocWords();
  bool writeWords=_words;
  word wordsStored=0;
  hasher=0;
  SeqPos bit=WORDSIZE;
  word w=0;
  pair<SeqPos,SeqPos> region(0,0);
  for(SeqPos i=0;i<_length;i++){
    bit-=2;
    SeqPos inpi=reverse ? (_length-i-1):i;
    switch(str[inpi]){
    case '1': w|=((word)3<<bit);isPattern=true;_hashLength++;break;
    case '0':isPattern=true;break;
    default: throw MurasakiException("Invalid pattern character (dec:"+dstring(inpi)+" char: "+str[inpi]+") encountered at position "+dstring(inpi));
    }

    if(bit==0){
      if(writeWords)
	_words[wordsStored++]=(reverse ? ~w:w);
      bit=WORDSIZE;
      w=0;
    }
  }

  if(bit<WORDSIZE)
    _words[wordsStored++]=highN(WORDSIZE-bit) & (reverse ? ~w:w);
}

BitSequence::BitSequence(SequenceFileReader& reader,Sequence *s) :
  shmUsed(SHM_NONE),
  _hashLength(0),
  isPattern(false),
  seq(s),_words(0),
  reverse(false),
  counted(0)
{
  _length=SequenceFileReader::getLength(s->filename,opt.seqreadOptions);
  bit_count=_length*2;
  word_count=((bit_count+WORDSIZE-1)/(WORDSIZE)); //rounds up to nearest full word
  _words=allocWords();
  bool writeWords=_words;
#ifdef MURASAKI_MPI
  if(seq && mpi_capable && !mpi_isHostLeader)
    writeWords=false;
#endif
  word wordsStored=0;
  hasher=0;
  SeqPos counters[4]={0,0,0,0};
  SeqPos bit=WORDSIZE;
  word w=0;
  bool inRegion=false;
  bool matchable=false;
  pair<SeqPos,SeqPos> region(0,0);
  for(SeqPos i=0;i<_length;i++){
    bit-=2;
    char c=reader.getc();
    switch(c){
    case 'C':case 'c': counters[0]++;counted++;matchable=true;break;
    case 'A':case 'a': w|=((word)1<<bit);counters[1]++;counted++;matchable=true;break;
    case 'T':case 't': w|=((word)2<<bit);counters[2]++;counted++;matchable=true;break;
    case 'G':case 'g': w|=((word)3<<bit);counters[3]++;counted++;matchable=true;break;
    case 'N':case 'n': matchable=false; break;
    case 0: //eof
      throw MurasakiDebugException("Fewer characters in stream than predicted ("+dstring(i)+" read vs "+dstring(_length)+") predicted!");
    default: throw MurasakiException("Non ACGT character (dec:"+dstring(c)+" char: "+c+") encountered at position "+dstring(i));
    }
    if(inRegion){ //in a matchable zone
      if(!matchable){//region end!
	region.second=i-1;
	inRegion=!inRegion;
	matchRegions.push_back(region);
      }
    }else{ //was in an N zone
      if(matchable){//region start
	region.first=i;
	inRegion=!inRegion;
      }
    }

    if(bit==0){
      if(writeWords)
	_words[wordsStored++]=(reverse ? ~w:w);
      bit=WORDSIZE;
      w=0;
    }
  }

  if(inRegion){
    region.second=_length-1;
    matchRegions.push_back(region);
  }

  if(bit<WORDSIZE)
    if(writeWords){
      _words[wordsStored++]=highN(WORDSIZE-bit) & (reverse ? ~w:w);
    }
  if(writeWords){
    assert(wordsStored==word_count);
    cout << "Collected "<<wordsStored<<" words (size: "<<word_count<<")"<<endl;
  }
  else {
    cout << "Postponed loading of "<<word_count<<" words (will fetch them directly later)"<<endl;
  }
  cout << "Read "<<counted<<"bp ("<<humanMemory(counted)<<")"<<endl;
  //set base frequency
  globalCounted+=counted;
  for(int b=0;b<4;b++){
    globalBaseCount[b]+=counters[b];
    //    globalBaseFreq[b]+=(double)counters[b]/(double)counted/2;
  }

#ifdef USE_SHM_MMAP
  if(shmUsed==SHM_MMAP_RW)
    mmap_msync();
#endif
}

BitSequence::BitSequence(const word length,Sequence *s,bool _reverse) :
  shmUsed(SHM_NONE),
  _length(length),bit_count(length*2),
  word_count((bit_count+WORDSIZE-1)/(WORDSIZE)), //rounds up to nearest full word)
  _hashLength(0),isPattern(false),
  seq(s),
  _words(0),
  reverse(_reverse),
  counted(0)
{
  _words=allocWords();
}

BitSequence* BitSequence::reverseComplement(){
  BitSequence* rev=new BitSequence(_length,seq,!reverse);
  bool writeWords=!opt.reverseOtf;
#ifdef MURASAKI_MPI
  if(seq && mpi_capable && !mpi_isHostLeader)
    writeWords=false;
#endif
  if(writeWords){
    int bitOffset=MODWORDSIZE(bit_count);
    for(word *src=_words+word_count-1,*dst=rev->_words;src>=_words;--src,++dst){
      *dst=revCompWord(*src);
      if(bitOffset){//have to shuffle bits
	if(dst!=rev->_words){ //if this wasn't my first, i have to fix the one before me too
	  *(dst-1)|=*dst>>bitOffset;
	}
	*dst<<=WORDSIZE-bitOffset;
      }
    }

    //and if we're doing the mmap grunt work, we have to write this out.
#ifdef USE_SHM_MMAP
    if(shmUsed==SHM_MMAP_RW && !opt.reverseOtf)
      rev->mmap_msync();
#endif
  }

  //have to flip regions too
  rev->matchRegions.reserve(matchRegions.size());
  for(vector<pair<SeqPos,SeqPos> >::iterator i=matchRegions.begin();i!=matchRegions.end();++i)
    rev->matchRegions.push_back(pair<SeqPos,SeqPos>(_length-i->second-1,_length-i->first-1));
  
  std::reverse(rev->matchRegions.begin(),rev->matchRegions.end()); //oops. i eclipsed std::reverse with my variable name.

  globalCounted+=counted; //add this to globalCounted or we'll get confused about our sequence lengths.
  return rev;
}

void BitSequence::mmap_msync(){
#ifdef USE_SHM_MMAP
  assert(shmUsed==SHM_MMAP_RW);
  cout << "Flushing mmap'd region to disk."<<endl;
  int res=msync(_words,word_count*sizeof(word),MS_SYNC);
  if(res<0){
    int err=errno;
    throw MurasakiException(string("Error synching mmap'd sequence to disk")+string(strerror(err)));
  }
#else
  throw MurasakiDebugException("mmap_msync called without mmap support??");
#endif
}

void finishGlobalFrequencies(){
  const char *forwardBases[4]={"C","A","T","G"};
  const char *bases[4]={"C/G","A/T","T","G"};
  cout << "Forward frequencies:";
  word baseTotal=0;
  for(int b=0;b<4;b++)baseTotal+=globalBaseCount[b];
  for(int b=0;b<4;b++)
    cout << "  "<<forwardBases[b]<<":"<<(double)globalBaseCount[b]/(double)baseTotal*100<<"%";
  cout << endl;
  for(int b=0;b<2;b++){
    globalBaseFreq[b]=((double)(globalBaseCount[b]+globalBaseCount[3-b]))/(double)baseTotal;
    globalBaseFreq[3-b]=globalBaseFreq[b];
  }
  cout << "Global base frequencies: "<<endl;
  for(int b=0;b<2;b++){
    cout << "   "<< bases[b] << ":" << globalBaseFreq[b]*100 << "%" << endl;
  }
}

void BitSequence::init(const BitSequence& a){
  _words=a._words;
  word_count=a.word_count;
  bit_count=a.bit_count;
  _length=a._length;
  _hashLength=a._hashLength;
  hasher=a.hasher;
}

//constructors
BitSequence::BitSequence(const string &str,Sequence *s,bool _reverse):
  shmUsed(SHM_NONE),isPattern(false),
  seq(s),
  _words(0),reverse(_reverse),
  counted(0)
{ //from 0101-ish patterns (as mask: ie double each bit)
  initPattern(str);
}

BitSequence::BitSequence(const BitSequence& a) :  //copy
  matchRegions(a.matchRegions),
  shmUsed(a.shmUsed),
  _length(a._length),
  bit_count(a.bit_count),
  word_count(a.word_count),
  _hashLength(a._hashLength),
  isPattern(a.isPattern),
  seq(a.seq),
  _words(a._words),
  reverse(a.reverse),
  counted(a.counted),
  hasher(a.hasher)
{
  assert(cerr << "Alert: Copying a bitsequence."<<endl);
}

BitSequence::BitSequence() : //uninited
  _length(0),
  bit_count(0),word_count(0),
  _hashLength(-1),isPattern(false),seq(0),_words(0),reverse(false),
  counted(0)
{}

BitSequence::~BitSequence() {
  if(_words){  //if a sequence got inited and never used, it's possible it didn't get _words
    if(reverse && opt.reverseOtf)return; //not our job to cleanup.
    switch(shmUsed){
    case SHM_NONE:
      delete[] _words;
      break;
    case SHM_MMAP_RO:case SHM_MMAP_RW:
#ifdef USE_SHM_MMAP
      munmap(_words,word_count*sizeof(word));
#endif
      break;
    case SHM_SYSV:
#ifdef USE_SHM_SYSV
      if(reverse)
	break; //nothing to do. w00t!
      //not really critical, but we can detach the sysv pages...
      if(shmdt(_words)<0)
	throw MurasakiException("Couldn't detach System V shared memory region for "+(seq ? seq->filename:string("(anon)"))+strerror(errno));
#endif
      break;
    default:
      assert(cerr << "Impossible case." << endl && 0);
    }
  }
}

int HashMethod::maxSources(){
  vector<int> members;
  vector<int> tmp;
  transform(inputs.begin(),inputs.end(),back_insert_iterator<vector<int> >(tmp),select1st<pair<int,int> >());
  sort(tmp.begin(),tmp.end());
  unique_copy(tmp.begin(),tmp.end(),back_insert_iterator<vector<int> >(members));
  int score=0;
  for(vector<int>::iterator i=members.begin();i!=members.end();i++){
    if(!popmap.count(*i)) //new to the area?
      score+=(popmap[*i]=popCount(seq->readWord(*i)));
    else
      score+=popmap[*i];
  }
  return score;
}

int HashMethod::entropyCheck(){
  //calculate entrop based on pattern and inputs
  int targetBits=(hash_bits+1)/2;
  entropy.clear();used.clear();
  entropy.resize(targetBits,0);
  used.resize(seq->length(),0);
  active=0;
  for(InputList::iterator i=inputs.begin();i!=inputs.end();i++){
    word w=seq->readWord(i->first);
    if(i->second > 0)
      w>>=i->second*2;
    else
      w<<=-i->second*2;
    for(unsigned b=0;b<entropy.size();b++){
      int srcbit=i->first*WORDSIZE/2+WORDSIZE/2-b-i->second-1;
      if(w & (3<<(b*2))){
	entropy[b]++;
	active|=3<<(b*2);
	if(srcbit<seq->length()) //only count stuff up till end of pat
	  used[srcbit]++;
      }
    }
  }
  
  sources=0;
  for(unsigned i=0;i<used.size();i++)
    if(used[i])
      sources++;
  assert(sources<=seq->hashLength());

  //evaluate entropy
    int sum=0,hi=entropy.front(),lo=entropy.front();
  empties=0;
  for(vector<int>::iterator i=entropy.begin();i!=entropy.end();i++){
    if(*i)
      sum+=*i;
    else
      empties++;
    hi=max<int>(hi,*i);
    lo=min<int>(lo,*i);
  }
  return sum-empties*seq->_hashLength //empties are evil
    -(hi-lo) //drawing all inputs from one source is not a good thing
    +(seq->word_count)*(targetBits-inputs.size()) //bulky outputs are bad
    +seq->_hashLength*(sources-hash_bits); //stricter enforcement of above rule
}

void BitSequence::compileHashFunc() {
  assert(isPattern);
  list<HashMethod*>  children;
  vector<int> score;
  for(int c=0;c<10;c++){ //spawn 10 kids
    children.push_back(new HashMethod(this));
  }
  
  pair<int,list<HashMethod*>::iterator > best,
    worst(children.front()->score,children.begin());
  int cycle;
  int improved=0;
  for(cycle=0;cycle<5000 || (cycle-improved)<(cycle>500 ? cycle/10:50);cycle++){ //arbitrarily large!
    //reset best/worst
    worst.first=best.first=children.front()->score;
    worst.second=best.second=children.begin();  
    //cout << "Starting iteration "<<cycle<<endl;
    for(list<HashMethod*>::iterator i=children.begin();
	i!=children.end();i++){
      if((*i)->score>best.first){
	best.first=(*i)->score;
	best.second=i;
	improved=cycle;
      }
      if((*i)->score<=worst.first){ //<= makes sure best!=worst
	worst.first=(*i)->score;
	worst.second=i;
      }
    } //resolved best and worst kid
    //    cout << "Best: "<<best.first<<" id:"<<(*best.second)->id<<" "<<**best.second<<endl;
    //    cout << "Worst: "<<worst.first<<" id:"<<(*worst.second)->id<<" "<<**best.second<<endl;

    //do we have a winner?
    if(best.first>0)
      goto CLEANUP; //oh god a goto, the joys

    assert(worst.second!=best.second);
    //kill the worst kid >_<;;
    //    cout << "Kill worst child: "<<(*worst.second)->id<<" score: "<<worst.first<<endl;
    delete *worst.second;
    children.erase(worst.second);
    //clone the best kid
    children.push_back(new HashMethod(**best.second));
    children.push_back(new HashMethod(**best.second));
    children.push_back(new HashMethod(**best.second));
    //mutate all but the best
    for(list<HashMethod*>::iterator i=children.begin();
	i!=children.end();i++)
      if(i!=best.second)
	(*i)->mutate();
    //    cout << "Done mutating."<<endl;
  }
 CLEANUP:
  cout << "Acceptable hash function found after "<<cycle-1<<" cycles."<<endl;
  hasher=*best.second;//keep the best
  for(list<HashMethod*>::iterator i=children.begin();
      i!=children.end();i++)
    if(i!=best.second)
      delete *i; //kill the rest
  hasher->finalize();
  hasher->entropyCheck();
  cout << "Best uses "<<hasher->sources<<" sources and has "<<hasher->empties<<" empties in "<<hasher->inputs.size()<<" inputs"<<endl;
  //  cout << "Pre cleanup: "<<*hasher<<endl;
  cout << "Post cleanup: "<<*hasher<<endl;
}

void HashMethod::finalize(){
  removeDuplicates();
  pruneUseless();
}

void HashMethod::removeDuplicates(){
  InputList tmp;
  sort(inputs.begin(),inputs.end());
  unique_copy(inputs.begin(),inputs.end(),insert_iterator<InputList>(tmp,tmp.begin()));
  inputs=tmp;
}

ostream& operator<<(ostream &os,const HashMethod &a){
  HashMethod::InputList::const_iterator last=a.inputs.end();
  last--;
  for(HashMethod::InputList::const_iterator i=a.inputs.begin();i!=a.inputs.end();i++){
    if(i->second > 0)
      os << "w["<<i->first<<"]>>"<<i->second;
    else if(i->second < 0)
      os << "w["<<i->first<<"]<<"<<0-i->second;
    else 
      os << "w["<<i->first<<"]";
  if(i!=last)
    os << " ^ ";
  }
  os << " {"<<a.inputs.size()<<"}";
  return os;
}

int HashMethod::align(int w){
  word pat=seq->readWord(w);
  //try all possible alignments
  multimap<int,int> score2shift;
  int bestscore=0;
  for(int shift=1-WORDSIZE/2;shift<WORDSIZE/2;shift++){
    int score;
    if(shift>0)
      score=popCount((pat>>shift*2) & ~active & hash_mask);
    else
      score=popCount((pat<<-shift*2) & ~active & hash_mask);
    bestscore=max<int>(score,bestscore);
    score2shift.insert(pair<int,int>(score,shift));
  }
  //choose randomly from among the highest scoring options
  vector<int> options;
  transform(score2shift.lower_bound(bestscore),score2shift.end(),
	    insert_iterator<vector<int> >(options,options.begin()),
	    select2nd<map<int,int>::value_type>());
  return options[random()%options.size()];
}

void HashMethod::pruneUseless(){
  vector<InputList::iterator> toErase;
  for(InputList::iterator i=inputs.begin();i!=inputs.end();i++){
    word preview=seq->readWord(i->first);
    if(i->second>=0)
      preview>>=i->second*2;
    else
      preview<<=-i->second*2;
    preview&=hash_mask;
    if(!popCount(preview))
      toErase.push_back(i);
  }
  for(vector<InputList::iterator>::iterator i=toErase.begin();
      i!=toErase.end();i++)
    inputs.erase(*i);
}

void HashMethod::mutate(){
  //  cerr << "Mutating "<<id<<" "<<*this<<" ";
  int fate=random()%100; //magical 100-sided mutation die
  if(fate<10 || maxSources()<hash_bits){//randomly add another word
    vector<int> unused;
    vector<bool> used(seq->word_count,false);
    int w; //new word is wehre?
    for(int i=0;i<(int)inputs.size();i++) //find out what's been used
      used[inputs[i].first]=true;; //incidentally all this could be kept between calls
    for(word i=0;i<seq->word_count;i++)
      if(!used[i])
	unused.push_back(i);
    if(unused.empty()) //add another word at random if boned
      w=random()%seq->word_count;
    else //if possible add a new word
      w=unused[random()%unused.size()];
    inputs.push_back(pair<int,int>(w,align(w))); //this could be done better. target a random unused bit
  }else if(fate<20 && inputs.size()>1){ //randomly kill a word
    inputs.erase(inputs.begin()+random()%inputs.size());
  }else{
    //bump a random word in a random direction
    int w=random()%inputs.size();
    int shift=inputs[w].second+random()%2 ? 1:-1;
    if(shift>=WORDSIZE/2)
      shift-=WORDSIZE; //wraparound
    else if(shift<=-WORDSIZE/2)
      shift+=WORDSIZE;
    inputs[w].second=shift;
  }
  //  removeDuplicates();
  score=entropyCheck();
  //   cerr << "->" << *this<< " new score: "<<score<<endl;
}

HashMethod::HashMethod(HashMethod &a) :
  inputs(a.inputs),seq(a.seq),score(a.score),id(nextId++),
  entropy(a.entropy),used(a.used),sources(a.sources),empties(a.empties),
  active(a.active)
{
  //  cout << a.id << " cloned => "<<id<<" ("<<*this<<")"<<endl;
}

HashMethod::HashMethod(BitSequence *s) :
  seq(s),id(nextId++),active(0)
{
  //  cout << "Brand new child "<<id;
  cout.flush();
  int w=random()%s->word_count;
  inputs.push_back(pair<int,int>(w,align(w))); //every kid starts on a random word
  score=entropyCheck();
  //  cout << " score: "<<score<<endl;
}

bool operator==(const BitSequence& a,const BitSequence& b){
  for(word i=0;i<a.word_count && i<b.word_count;i++)
    if(a.readWord(i)!=b.readWord(i))return false;
  return true;
}

Location::Location(Sequence *s,SeqPos p) :
  seqno(lookup_seqtable(s)),pos(p) { }

Location::Location(SeqIdx s,SeqPos p) :
  seqno(s),pos(p) { }

Location::Location() :
  seqno(255),pos(0) { } //this should only be used to create temp storage

Sequence* Location::seq() const{
  return seq_table[seqno];
}

BitSequence* Location::bitSeq(){return pos>0 ? seq()->fwd:seq()->rev;}

word Hash::sizeAt(const HashKey key,const HashVal &a){
  //universally functionall sizeAt...
  LocList locs(seq_count);
  this->lookup(key,a,locs);
  unsigned high=0;
  for(LocList::iterator i=locs.begin();i!=locs.end();i++){
    high=max<unsigned>(i->size(),high);
  }
  return high;
}

void Hash::dump(ostream &os){
  HashKey key;
  unsigned s;

  for(key=0; key<(unsigned)hash_size; key++){
    unsigned c=0;
    LocList  l(seq_count); lookup(key, l);    

    s = sizeAt(key);
    if( s==0 ) continue;

    os.write( (char*)&key, sizeof(HashKey)  );
    os.write( (char*)&s,   sizeof(unsigned) );
    //    cout << "dump: size=" << s<< endl;
    for(unsigned i=0; i<l.size(); i++){
      for(list<Location>::iterator ll=l.at(i).begin(); ll!=l.at(i).end(); ll++){
	Location lll=*ll;
	os.write( (char*)&lll, sizeof(Location) );
	c++;
      }
    }
    if (s!=c){ cerr << "Hash::dump(): incorrect size?" << endl; }
  }

  key = 0;
  s = 0;
  os.write( (char*)&key, sizeof(HashKey)  );
  os.write( (char*)&s,   sizeof(unsigned) );
}

void Hash::load(istream &is){
  HashKey key;
  unsigned s;
  Location l;
  Location *buf;

  do {
    is.read( (char*)&key, sizeof(HashKey)  );
    is.read( (char*)&s,   sizeof(unsigned) );
    if( key==0 && s==0 ) break;

    buf = (Location*)malloc(sizeof(Location) * s);
    is.read( (char*)buf, sizeof(Location) * s);
    for(unsigned i=0; i<s; i++) add(key, buf[i]);
    free(buf);
  } while(true);
}

Hash::Hash(BitSequence *pat) :
  hashpat(pat)
{
}

Hash::~Hash(){}

void Hash::writeHistogram(ostream &of){
  vector<SeqPos> histo(1,0);
  for(word base=0;base<(word)hash_size;base++){ //oh this is fun...
    if(emptyAt(base))
      continue;
    usedBuckets++;
    unsigned size=sizeAt(base);
    if(size>=histo.size())
      histo.resize(size+1);
    histo[size]++;
  }
  if(of!=cerr)
    for(unsigned i=1;i<histo.size();i++){
      of << i << "\t" << histo[i]<<endl;
    }
}

void Hash::writeDetailedHistogram(ostream &of){
  for(word base=0;base<(word)hash_size;base++){ //oh this is fun...
    if(emptyAt(base))
      continue;
    unsigned size=sizeAt(base);
    if(size){
      if(opt.histogram>3){ //super freaking detailed
	LocList ll(seq_count);
	lookup(base,ll); //fetch all locations...
	vector<Location> locs;
	back_insert_iterator<vector<Location> > ii(locs);
	for(int i=0;i<seq_count;i++) 
	  copy(ll[i].begin(),ll[i].end(),ii); //throw all into big list
	sort(locs.begin(),locs.end());
	vector<Location>::iterator i=locs.begin();
	Location last(*i);
	word count=1;
	usedPats++; //we got at least 1 new one
	do{
	  i++;
	  if(i==locs.end())
	    break;
	  Location here(*i);
	  if(last<here){ //different src pattern
	    of << base << "\t" << Window(last,hashpat).prettyString() << "\t" << count << endl;
	    if(count>worstPat)
	      worstPat=count;
	    count=1;
	    last=here;
	    usedPats++;
	  }
	  count++;
	}
	while(1);
	if(count>worstPat)
	  worstPat=count;
	of << base << "\t" << Window(last,hashpat).prettyString() << "\t" << count << endl;
      }else
	of << base<<":"<<size << endl;
    }
  }
}

UsedMap::UsedMap():
  used(seq_count)
{ }

bool UsedMap::sanityCheck() {
  for(unsigned i=0;i<used.size();i++){
    for(usedItree::iterator s=used[i].begin();
	s!=used[i].end();s++){
      assert(*s);
      assert((*s)->sanityCheck());
    }
  }
  return 1;
}

bool AnchorSet::sanityCheck() const {
  for(unsigned i=0;i<spaces.size();i++){
    assert(UsedInt_sanityCheck(spaces[i].key()));
    assert(*spaces[i]==this);
  }
  return 1;
}

bool IntervalSet::sanityCheck() const {
  for(unsigned i=0;i<spaces.size();i++){
    assert(UsedInt_sanityCheck(spaces[i]));
  }
  return 1;
}


bool Hash::sanityCheck(){
  for(word base=0;base<(word)hash_size;base++){
    if(emptyAt(base))
      continue;
    LocList loclist;
    lookup(base,loclist);
    for(unsigned s=0;s<loclist.size();s++)
      for(LocSubList::iterator i=loclist[s].begin();
	  i!=loclist[s].end();i++){
	Window w(*i,hashpat);
	word here=w.hash();
	//      cout << "hash for "<<l<<" ("<<w<<"): "<<wordToString(here) << " (vs "<< wordToString(base)<<")"<<endl;
	if(here!=base){
	  assert(here==base);
	  return false;
	}
      }
  }
  return true;
}

bool Hash::sanityCheck(word base){
  if(emptyAt(base))
    return true;
  //  cout << "Spot checking "<<wordToString(base)<<endl;
  LocList loclist;
  lookup(base,loclist);
  for(unsigned s=0;s<loclist.size();s++)
    for(LocSubList::iterator i=loclist[s].begin();
	i!=loclist[s].end();i++){
      Window w(*i,hashpat);
      word here=w.hash();
      //    cout << "  ** hash for "<<l<<" ("<<w<<"): "<<wordToString(here)<<endl;
      if(here!=base){
	assert(here==base);
      return false;
      }
    }
  return true;
}

bool UsedMap::fetchSetIds(int seq,const UsedInt &a,set<AnchorSet*> &out){
  bool res=false;
  for(usedItree::range_iterator i(used[seq].in_range(a));i!=used[seq].end();++i){
    out.insert(*i);
    res=true;
  }
  return res;
}

bool UsedMap::fetchNearbySetIds(int seq,const UsedInt &a,set<AnchorSet*> &out){
  bool res=false;
  for(usedItree::range_iterator i(used[seq].in_range(opt.joinNeighbors ? UsedInt_grown(a,opt.joinDist):a));
      i!=used[seq].end();++i){
    out.insert(*i);
    res=true;
  }
  return res;
}

bool UsedMap::alreadyExists(const IntervalSet& a){
  set<AnchorSet*> matches,temp,context;

  int i=0;
  while(isZero(a.spaces[i])){
    i++;
    assert(i<seq_count);
  }

  if(!fetchSetIds(i,a.spaces[i],context))
    return false;

  for(i++;i<seq_count;i++){
    if(isZero(a.spaces[i]))
      continue;
    if(!fetchSetIds(i,a.spaces[i],temp))
      return false;
    set_intersection(temp.begin(),temp.end(),
		     context.begin(),context.end(),
		     inserter(matches,matches.begin()));
    if(matches.empty())
      return false;
    context.clear();
    context.insert(matches.begin(),matches.end());
    matches.clear();
  }
  bool winner=false;
  //must find out if a is entirely inside of anything in context
  for(set<AnchorSet*>::iterator ite=context.begin();
      ite!=context.end();
      ite++){
    AnchorSet *s=*ite;
    if(s->contains(a)){
      winner=true;
      if(opt.retainMembers){ //if we're keeping track of members, this counts as a hit.
	for(HashCount::const_iterator i=a.members.begin();i!=a.members.end();i++)
	  s->members[i->first]+=i->second;
      }
    }
  }
  return winner;
}

bool UsedMap::merge(IntervalSet& a){
  //  assert(a.sanityCheck());
  set<AnchorSet*> matches,temp,context;

  int i=0;
  while(isZero(a.spaces[i])){
    i++;
    assert(i<seq_count);
  }

  if(!fetchNearbySetIds(i,a.spaces[i],context))
    return false;

  for(i++;i<seq_count;i++){
    if(isZero(a.spaces[i]))
      continue;
    if(!fetchNearbySetIds(i,a.spaces[i],temp))
      return false;
    set_intersection(temp.begin(),temp.end(),
		     context.begin(),context.end(),
		     inserter(matches,matches.begin()));
    if(matches.empty())
      return false;
    context.clear();
    context.insert(matches.begin(),matches.end());
    matches.clear();
  }
  //context now contains all anchorsets which a overlaps in each sequence individually
  //(but any single anchorset might not overlap in all sequences!!!)
  assert(a.sanityCheck());
  for(set<AnchorSet*>::iterator i=context.begin();
      i!=context.end();
      i++){
    if(!a.overlaps(**i))
      continue;
    assert((*i)->sanityCheck());
    //    cout << "Merging: "<<a<<" with "<< **i<<endl;
    //    assert(sanityCheck());
    assert(a.sanityCheck());
    a.coalesce(**i);
    //    assert(sanityCheck());
    assert(a.sanityCheck());
    remove(*i);
    //    cout << "Result: "<<a<<endl;
    //    assert(sanityCheck());
    assert(a.sanityCheck());
  }
  add(a);
  return true;
}

void UsedMap::add(const IntervalSet &a){
  //  cout << "UsedMap::Add("<<const_cast<IntervalSet&>(a)<<")"<<endl;
  new AnchorSet(a,*this); //add it
}

void UsedMap::insert(IntervalSet& a){
  //  assert(cout << "Insert: "<<a<<endl);
  if(alreadyExists(a)){
    //    cout << "--- Already exists."<<endl;
    return; //already complete inside some other anchor?
  }

  //if not, extend it.
  a.extend();
  //  assert(cout << "After extend: "<<a<<endl);
  if(opt.dumpEntropy){//entropy check?
    pair<double,SeqPos> entropy(a.entropy());
    *entropy_fh << entropy.first << "\t" << entropy.second << endl;
  }

  if(merge(a)){ //try to merge it
    //    cout << "--- Merged"<<endl;
    return; 
  }
  add(a); //completely new and fresh!
  //  cout << "--- Added as a new anchor"<<endl;
}

void UsedMap::remove(AnchorSet *a){
  //  assert(sanityCheck());
  for(unsigned i=0;i<a->spaces.size();i++){
    //    assert(cerr << "Erasing seq "<<i<<" - "<<*(a->spaces[i])<<" from: "<<used[i]<<endl);
    used[i].erase(a->spaces[i]);
    //    assert(sanityCheck());
  }
  //  assert(sanityCheck());
  delete a;
  //  assert(sanityCheck());
}

bool IntervalSet::contains(const IntervalSet &a){
  for(unsigned i=0;i<spaces.size();i++){
    if(isZero(a.spaces[i]) && isZero(spaces[i]))
      continue;
    if(isZero(a.spaces[i]) != isZero(spaces[i]))
      return false;
    if(!UsedInt_contains(spaces[i],a.spaces[i]))
      return false;
  }
  return true;
}

bool AnchorSet::contains(const IntervalSet &a){
  for(unsigned i=0;i<spaces.size();i++){
    if(isZero(a.spaces[i]) && isZero(spaces[i]))
      continue;
    if(isZero(a.spaces[i]) != isZero(spaces[i]))
      return false;
    if(!UsedInt_contains(spaces[i].key(),a.spaces[i]))
      return false;
  }
  return true;
}

bool IntervalSet::overlaps(const IntervalSet &a){
  for(unsigned i=0;i<spaces.size();i++){
    if(isZero(a.spaces[i]) && isZero(spaces[i]))
      continue;
    if(isZero(a.spaces[i]) != isZero(spaces[i]))
      return false;
    if(!spaces[i].overlaps(a.spaces[i]))
      return false;
  }
  return true;
}

bool IntervalSet::overlaps(const AnchorSet &a){
  for(unsigned i=0;i<spaces.size();i++){
    if(isZero(a.spaces[i]) && isZero(spaces[i]))
      continue;
    if(isZero(a.spaces[i]) != isZero(spaces[i]))
      return false;
    if(!spaces[i].overlaps(a.spaces[i].key()))
      return false;
  }
  return true;
}

bool UsedInt_contains(const UsedInt &outer,const UsedInt &inner){
  return outer.start<=inner.start && outer.stop>=inner.stop;
}

void IntervalSet::coalesce(const AnchorSet &a){
  for(size_t i=0;i<spaces.size();i++){
    if(isZero(a.spaces[i]) || isZero(spaces[i]))
      continue;
    UsedInt_coalesce(spaces[i],a.spaces[i].key());
  }
  //mmmm tasty (metadata - nom nom!)
  if(opt.retainMembers)
    for(HashCount::const_iterator i=a.members.begin();i!=a.members.end();i++)
      members[i->first]+=i->second;
}

void UsedInt_coalesce(UsedInt &x,const UsedInt &a){
  if(isZero(x) || isZero(a))
    return;
  x.start=x.start<=a.start ? x.start:a.start;
  x.stop=x.stop>=a.stop ? x.stop:a.stop;
}

UsedInt UsedInt_coalesced(const UsedInt &a,const UsedInt &b){
  assert(!isZero(a));  assert(!isZero(b));  //no forcing coalesces of non-intervals
  return UsedInt(a.start<=b.start ? a.start:b.start, a.stop>=b.stop ? a.stop:b.stop);
}

inline UsedInt UsedInt_grown(const UsedInt &a,const SeqPos amt){
  if(isZero(a)) //no growing non-intervals
    return a;

  if(a.start>0)
    return UsedInt(max<SeqPos>(a.start-amt,1),a.stop+amt);
  else
    return UsedInt(a.start-amt,min<SeqPos>(a.stop+amt,-1));
}

UsedInt UsedInt_inverted(const UsedInt &a,const Sequence& s){
  if(isZero(a))
    return a;
  SeqPos length=s.length();
  if(a.start>0) //ugh. that took way more reverse engineering than it should have.
    return UsedInt(0-(length-a.start+1),0-(length-a.stop+1));
  else
    return UsedInt(a.start+length+1,a.stop+length+1);
}

UsedInt& UsedInt_invert(UsedInt &a,const Sequence& s){
  if(isZero(a))
    return a;
  SeqPos length=s.length();
  if(a.start>0){ //ugh. that took way more reverse engineering than it should have.
    a.start=0-(length-a.start+1);
    a.stop=0-(length-a.stop+1);
  }else{
    a.start=a.start+length+1;
    a.stop=a.stop+length+1;
  }
  return a;
}

bool UsedInt_sanityCheck(const UsedInt &a){
  assert(a.start <= a.stop);
  return true;
}

AnchorSet::AnchorSet(const IntervalSet &a,UsedMap &usedMap) :
  members(a.members) //if opt.retainMembers is off, this should be empty anyway
{
  spaces.reserve(seq_count);
  for(size_t i=0;i<a.spaces.size();++i)
    spaces.push_back(usedMap.used[i].insert(a.spaces[i],this));
}

void IntervalSet::add(HashVal &a,SeqPos length){
  length--;
  if(a.pos)
    spaces.push_back(UsedInt(a.pos>0 ? a.pos:a.pos-length,
			     a.pos>0 ? a.pos+length:a.pos));
  else //it's a blank in an island
    spaces.push_back(UsedInt(0,0));
  //  assert(cout << "Added "<<spaces.back()<<" to interval set "<<*this<<endl);
}

SeqPos length(const UsedInt& a){
  return a.stop-a.start+1;
}

bool operator<(const Location &a,const Location &b){
  Location copya(a),copyb(b);
  Window wina(copya,mfh->hashpat),winb(copyb,mfh->hashpat);
  //  assert(cout << "Comparing "<<wina.prettyString()<<" to "<<winb.prettyString()
  //	 << " : "<< (wina<winb ? "<":">=")<<endl);
  //mm construct two temporary windows? delicious
  return wina<winb;
}

bool operator<(const Window &a,const Window &b){
  for(SeqPos frame=0;frame<a.frames;frame++){
    word pword=a.pat->readWord(frame);
    word resa=a.buffer[frame] & pword;
    word resb=b.buffer[frame] & pword;
    if(resa<resb)
      return 1;
    if(resb<resa)
      return 0;
  }
  return 0;
}

void IntervalSet::invert(){
  for(unsigned i=0;i<spaces.size();++i){
    UsedInt_invert(spaces[i],*seqs[i]);
  }
}

void IntervalSet::extend(){
  //  assert(cout << "Extend: "<<*this<<endl);
  int baseSeq=0;
  while(isZero(spaces[baseSeq])){
    baseSeq++;
    assert(baseSeq<seq_count);
  }
  assert(baseSeq<seq_count-1);
  BitSequence *mySeq=spaces[baseSeq].stop>0 ? seqs[baseSeq]->fwd :
    seqs[baseSeq]->rev,*tSeq;
  SeqPos mstart=bitSeqCoords(spaces[baseSeq].start),tstart;
  SeqPos mstop=bitSeqCoords(spaces[baseSeq].stop),tstop;
  SeqPos right=mySeq->length()-mstop,
    left=mstart;
  assert(mstart>=0);
  for(unsigned i=baseSeq+1;i<spaces.size();i++){
    if(isZero(spaces[i]))
       continue;
    tSeq=spaces[i].stop>0 ? seqs[i]->fwd :
      seqs[i]->rev;
    tstart=bitSeqCoords(spaces[i].start);
    tstop=bitSeqCoords(spaces[i].stop);
    if(spaces[i].stop<0)
      swap(tstart,tstop); //sneaky devils

    assert(tstart>=0);
    
    right=min<SeqPos>(tSeq->length()-tstop-1,min<SeqPos>(right,mySeq->cmpRight(mstop+1,*tSeq,tstop+1)));
    left=min<SeqPos>(tstart,min<SeqPos>(left,mySeq->cmpLeft(mstart-1,*tSeq,tstart-1)));
  }

  if(right || left){
    //    assert(cout << *this << "-> Extending "<<mstart<<","<<mstop<<" by "<<left<<","<<right<<" to ");
    assert(sanityCheck());
    if(right)
      for(unsigned i=0;i<spaces.size();i++){
	if(isZero(spaces[i]))
	  continue;
	if(spaces[i].stop>0)
	  spaces[i].stop+=right;
	else
	  spaces[i].start-=right;
      }
    if(left)
      for(unsigned i=0;i<spaces.size();i++){
	if(isZero(spaces[i]))
	  continue;
	if(spaces[i].start>0)
	  spaces[i].start-=left;
	else
	  spaces[i].stop+=left;
      }
    //    assert(cout << *this<<"<--"<<endl);
  }else{
    //    assert(cout << *this << "\\\\// Not extending"<<endl);
  }
  assert(sanityCheck());
}

bool operator==(const Location &a,const Location &b) {
  BitSequence *mySeq=a.pos>0 ? a.seq()->fwd : a.seq()->rev,
    *tSeq=b.pos>0 ? b.seq()->fwd : b.seq()->rev;
  SeqPos mstop=bitSeqCoords(a.pos),
    tstop=bitSeqCoords(b.pos);
  return mySeq->equal(mstop,*tSeq,tstop,*mfh->hashpat);
}

ostream& operator<<(ostream &os, const Window &a){
  for(SeqPos i=0;i<a.frames-1;i++){
    os << wordToString(a.buffer[i]) << " ";
  }
  os << "+" << wordToString(a.buffer[a.frames-1]);
  return os;
}

ostream& operator<<(ostream &os,const BitSequence &a){
  if(a.isPattern)
    return os<<a.asPattern();
  else
    return os << a.asString();
}

ostream& operator<<(ostream &os,const Location &a){
  return os << "Location: ("<< a.seq()->getId()<<","<<a.seq()->extCoords(a.pos)<<")";
}

void BitSequence::maskString(string &str) const{
  SeqPos j=0;
  for(word i=0;i<word_count;i++){
    word w=readWord(i);
    word mask=highN(2);
    while(mask){
      if(!(w & mask))
	str[j]='.';
      j++;
      mask>>=2;
    }
  }
}

string Window::toRawString(){
  string ret;
  for(unsigned i=0;i<buffer.size();i++)
    ret+=wordToString(buffer[i]);
  ret.resize(pat->length());
  return ret;
}

string Window::prettyString(){
  string ret(toRawString());
  pat->maskString(ret);
  size_t len=ret.length(),patlen=min<word>(pat->length(),hash_bits/2);
  switch(opt.quick_hash){
  case 2:
    for(size_t i=patlen;i<len;i++)
      ret[i]=tolower(ret[i]);
    break;
  case 1:
    for(size_t i=WORDSIZE/2;i<len;i++)
      ret[i]=tolower(ret[i]);
    break;
  case 3:
    for(size_t i=0;i<len;i++)
      if(!pat->hasher->used[i])
	ret[i]=tolower(ret[i]);
    break;
  }
  return ret;
}

void Window::slide(){
  pos+=2;
  SeqPos i;
  for(i=0;i<frames-1;i++){
    buffer[i]=(buffer[i]<<2) | (buffer[i+1]>>(WORDSIZE-2));
  }
  buffer[i]<<=2;

  //this should be done at the end. thus frame is always correct and no buffers are ever at empty
  if(pos/WORDSIZE!=frame){
    frame=pos/WORDSIZE;
    buffer[frames-1]=seq->readWord(frame+frames-1);
  }
}

void Window::slideWord(){
  //update to new positions
  pos+=WORDSIZE;
  frame++;

  SeqPos i;
  for(i=0;i<frames-1;i++){
    buffer[i]=buffer[i+1];
  }
  //[i] is now the last buffer
  buffer[i]=seq->readWord(frame+frames-1);
  SeqPos offset=MODWORDSIZE(pos);
  if(offset) //[i-1] needs to get the offset bits that had been chopped off of the previous [i]
    buffer[i-1]|=buffer[i]>>(WORDSIZE-offset);
  //and [i] needs to not duplicate those on a subsequent slide...
  buffer[i]<<=offset;
}

void Window::slide(SeqPos dist){ //bah. this function isnt actually used so, dont care that it's not as efficient as possible
  while(dist>WORDSIZE){
    slideWord();
    dist-=WORDSIZE;
  }

  while(dist>0){
    slide();
    dist-=2;
  }
}

void Window::initslide(SeqPos dist){
  assert(dist<=WORDSIZE-(MODWORDSIZE(pos)));
  pos+=dist;
  SeqPos i;
  for(i=0;i<frames-1;i++){
    buffer[i]=(buffer[i]<<dist) | (buffer[i+1]>>(WORDSIZE-dist));
  }
  buffer[i]<<=dist;

  if(pos/WORDSIZE!=frame){
    frame=pos/WORDSIZE;
    buffer[frames-1]=seq->readWord(frame+frames-1);
  }
}

bool Window::equals(const Window &a) const {
  assert(pat==a.pat); //dont be dumb
  if(pos+pat->bit_count>seq->bit_count) //overruns end of seq
    return false;
  for(SeqPos i=0;i<frames-1;i++){
    word result=(buffer[i] & pat->readWord(i)) ^ (a.buffer[i] & pat->readWord(i));
    if(result)
      return false; //a bit's amiss!
  }
  return true;
}

inline word Window::eqbases(const Window &a) const {
  return ~(buffer[0] ^ a.buffer[0]);
}

Window::Window(const UsedInt &a,const Sequence *s,BitSequence *p) :
  seq(a.start>0 ? s->fwd:s->rev),pat(p),pos(bitSeqCoords(a.start>0 ? a.start:a.stop)*2),
  frames(p->word_count+1),buffer(frames)
{
  SeqPos stop=pos;
  frame=pos/WORDSIZE;
  pos=frame*WORDSIZE;
  for(SeqPos i=0;i<frames;i++)
    buffer[i]=seq->readWord(frame+i);
  if(pos!=stop)
    initslide(MODWORDSIZE(stop)); //scuttle right the leftover bits
}

Window::Window(BitSequence *_seq,SeqPos _pos,BitSequence *_pat):
  seq(_seq),pat(_pat),pos(_pos*2),
  frames(_pat->word_count+1),buffer(frames)
{
  frame=pos/WORDSIZE;
  pos=frame*WORDSIZE;
  for(SeqPos i=0;i<frames;i++){
    buffer[i]=seq->readWord(frame+i);
  }
  if(pos!=_pos*2)
    initslide(MODWORDSIZE(_pos*2));//scuttle right the leftover bits
}

Window::Window(Location &v,BitSequence *p):
  seq(v.bitSeq()),pat(p),pos(bitSeqCoords(v.pos)*2),
  frames(p->word_count+1),buffer(frames)
{
  SeqPos stop=pos;
  frame=pos/WORDSIZE;
  pos=frame*WORDSIZE;
  for(SeqPos i=0;i<frames;i++)
    buffer[i]=seq->readWord(frame+i);
  if(pos!=stop)
    initslide(MODWORDSIZE(stop)); //scuttle right the leftover bits
}
  
word Window::hash(){
  word hash=0;
  word patpos=0;
  word filled=0,bit=WORDSIZE-2,frame=0;
  switch(opt.quick_hash){
  case 2: //naive hash: use first hash_bits bits
  while(filled<WORDSIZE && patpos<pat->bit_count){
    if(bit<0){
      frame++;
      bit=WORDSIZE-2;
    }
    if(pat->readCode(patpos)){
      hash=hash<<2 | ((buffer[frame]>>bit) & 3);
      filled+=2;
    }
    patpos+=2;
    bit-=2;
  }
  break;
  case 1: //use first word only (usually very bad waste of hash)
    hash=buffer[0] & pat->readWord(0);
    break;
  case 3: //adaptive hasher. frekain' smart (usually...)
    word patword=0;
    SeqPos lastpati=-1;
    for(HashMethod::InputList::iterator i=pat->hasher->inputs.begin();
	i!=pat->hasher->inputs.end();i++){
      if(i->first!=lastpati)
	patword=pat->readWord(i->first);
      if(i->second>=0)
	hash^=(buffer[i->first] & patword)>>i->second*2;
      else
	hash^=(buffer[i->first] & patword)<<-i->second*2;
      lastpati=i->first;
    }
    break;
  }

  return hash & hash_mask;
}

ostream& operator<<(ostream &os, const AnchorSet &a){
  return os << a.asString();
}

string UsedMap::asString() {
  string ret;
  for(usedItree::iterator i=used[0].begin();
      i!=used[0].end();i++){
    ret+= (*i)->asString() + "\n";
  }
  return ret;
}

ostream& operator<<(ostream &os,const pair<word,SeqPos> &p){
  return os << p.first << ":" << p.second;
}

ostream& UsedMap::writeTfidf(ostream &os){
  if(used[0].empty())
    return os;
  if(!dfCount)
    dfCount=makeDfCount();
  for(usedItree::iterator i=used[0].begin();
      i!=used[0].end();i++){
    assert((*i));
    AnchorSet *&s=*i;
    if(!opt.hitfilter || s->hitCount()>=opt.hitfilter)
      os << s->uniqueness(this) << endl;
  }
  return os;
}

ostream& UsedMap::saveDetails(ostream &os,ostream &bitos){
  if(used[0].empty())
    return os;
  if(opt.tfidf && !dfCount)
    dfCount=makeDfCount();
  for(usedItree::iterator i=used[0].begin();
      i!=used[0].end();i++){
    assert((*i));
    AnchorSet *&s=*i;
    if(!opt.hitfilter || s->hitCount()>=opt.hitfilter){
      os << s->asString() << "\t" << s->hitCount();

      if(opt.retainMembers){
	double score;
	if(opt.tfidf)
	  score=s->uniqueness(this);
	else {
	  pair<SeqPos,SeqPos> bscore(s->bitscore());
	  score=(double)bscore.first/(double)bscore.second;
	}
	os << "\t" << score << "\t";
	HashCount::iterator last=s->members.end();
	assert(!s->members.empty());
	last--;
	for(HashCount::iterator i=s->members.begin();i!=last;i++)
	  os << i->first << ":" << i->second <<",";
	os << last->first << ":" << last->second;
      }

      if(opt.bitscore && bitos!=cerr){
	pair<SeqPos,SeqPos> scores(s->bitscore());
	bitos << scores.first << "\t" << scores.second << endl;
      }
      os << endl;
    }
  }
  return os;
}

int UsedMap::count() const {
  return used[0].size();
}

ostream& UsedMap::writeOut(ostream &os){
  if(used[0].empty())
    return os;
  for(usedItree::iterator i=used[0].begin();
      i!=used[0].end();i++){
    assert(*i);
    if(!opt.hitfilter || (*i)->hitCount()>=opt.hitfilter)
      os << (*i)->asString() << "\n";
  }
  //as these output sets can get huge, this might be a bad idea
  //  return os << asString();
  return os;
}

word* UsedMap::makeDfCount(){
  size_t size;
  word *df;
  if(opt.tfidf){
    assert(!dfCount);
#ifdef MURASAKI_MPI
    if(mpi_capable)
      size=mpi_total_hash_size;
    else
#endif
      size=hash_size;

    cout << "(Creating df table for "<<size<<" entries...)"<<endl;
    df=new word[size];
    if(!df){ //i hope this works. or does failure actually throw an exception?
      cerr << "Warning: insufficient RAM for df table creation (disabling tfidf scoring)."<<endl;
      opt.tfidf=false;
      return df;
    }
    bzero(df,size*sizeof(word));

    if(count())
      for(usedItree::iterator i=used[0].begin();
	  i!=used[0].end();i++){
	assert(*i);
	AnchorSet *&s=*i;
	for(HashCount::iterator i=s->members.begin();i!=s->members.end();i++)
	  df[i->first]+=i->second;
      }
    return df;
  }
  return 0;
}

string AnchorSet::asString() const {
  string ret;
  for(unsigned i=0;i<spaces.size();i++){
    assert(*spaces[i]);
    ret+=asAnchor(spaces[i].key(),seqs[i]);
    if(i!=spaces.size()-1)
      ret+="\t";
  }
  return ret;
}

string asAnchor(const UsedInt& a,const Sequence *s){
  if(a.start<0) //gotta flip
    return dstring(s->extCoords(a.stop))+string("\t")+
      dstring(s->extCoords(a.start))+string("\t")+
      string(((a.start<0) != (a.stop<0)) ? "?":(a.start>=0 ? "+":"-"));
  else return dstring(s->extCoords(a.start))+string("\t")+
	 dstring(s->extCoords(a.stop))+string("\t")+
	 string(((a.start<0) != (a.stop<0)) ? "?":(a.start>=0 ? "+":"-"));
}

double AnchorSet::uniqueness(UsedMap *context) const {
  double sum=0;
#ifdef MURASAKI_MPI
  if(mpi_capable){
    if(!dfCount)dfCount=context->makeDfCount();
    if(dfCount)
      for(HashCount::const_iterator i=members.begin();i!=members.end();i++){
	sum+=i->second*log(anchors->count()/dfCount[i->first]);
      }
    return sum;
  }
#endif
  if(!opt.retainMembers){ //old way. takes into account non-anchored noise. not so cool
    for(HashCount::const_iterator i=members.begin();i!=members.end();i++){
      // wi = tfi*log(D/dfi) by the money
      if(!mfh->sizeAt(i->first)){
	throw MurasakiDebugException("What the monkey? "+dstring((long)i->first)+" isn't in hash?");
      }
      sum+=i->second*log((double)usedBuckets/(double)mfh->sizeAt(i->first));
    }
  }else{ //new way. separate counters. cleaner
    if(!dfCount)dfCount=context->makeDfCount();
    if(!dfCount)return 0;
    for(HashCount::const_iterator i=members.begin();i!=members.end();i++){
      if(!dfCount[i->first])
	throw MurasakiDebugException("dfCount missing at "+dstring((long)i->first)+" in anchor "+(this->asString()));
      sum+=i->second*log(anchors->count()/dfCount[i->first]);
    }
  }
  return sum;
}

pair<double,SeqPos> IntervalSet::entropy() const {
  double ret=0;
  vector<Window> windows;
  SeqPos shortest=0,shortlen=length(spaces[0]);
  for(int i=0;i<(int)spaces.size();i++){
    windows.push_back(Window(spaces[i],seqs[i],mfh->hashpat));
    if(shortlen>length(spaces[i])){
      shortest=i;
      shortlen=length(spaces[i]);
    }
  }

  int counters[4],offset;
  for(SeqPos pos=0;pos<shortlen;){
    for(offset=WORDSIZE-2;offset>=0 && pos<shortlen;pos++,offset-=2){
      memset(counters,0,sizeof(int)*4);
      
      for(uint s=0;s<spaces.size();s++)
	counters[(windows[s].firstWord()>>offset) & 3]++;
      
      //calculate (and add in) entropy for this column
      for(int b=0;b<4;b++)
	if(counters[b]){
	  double count=(double)counters[b];
	  ret+=count*log(count/globalBaseFreq[b]);
	}
    }
    for(vector<Window>::iterator ite=windows.begin();ite!=windows.end();ite++)
      ite->slideWord();
  }
  return pair<double,SeqPos>(ret,shortlen);
}

size_t AnchorSet::hitCount(){
  size_t ret=0;
  for(HashCount::iterator i=members.begin();i!=members.end();++i)
    ret+=i->second;
  return ret;
}

pair<SeqPos,SeqPos> AnchorSet::bitscore() const {
  vector<Window> windows;
  SeqPos shortest=0,shortlen=length(spaces[0].key());
  for(int i=0;i<(int)spaces.size();i++){
    windows.push_back(Window(spaces[i].key(),seqs[i],globalPat));
    if(shortlen>length(spaces[i].key())){
      shortest=i;
      shortlen=length(spaces[i].key());
    }
  }
  SeqPos eqbits=0;
  for(SeqPos pos=0;pos<shortlen;pos+=WORDSIZE/2){
    word ret=~0;
    //compare everything to ref seq. store in ret
    for(int comp=0;comp<(int)windows.size();comp++){
      if(comp==shortest)continue;
      word eq=windows[comp].eqbases(windows[shortest]);
      if(pos+WORDSIZE/2>length(spaces[comp].key()))
	eq=(eq & (highN(2*(length(spaces[comp].key())-pos))));
      eq=(((eq & alternating_mask)<<1) & eq);
      ret&=eq;
    }
    //count ret and add it up!
    eqbits+=popCount(ret);
    
    //shuffle everybody
    for(vector<Window>::iterator ite=windows.begin();ite!=windows.end();ite++)
      ite->slideWord();
  }
  return pair<SeqPos,SeqPos>(eqbits,shortlen);
}



string hashCacheName(string name, BitSequence *pat){
  ostringstream os;
  
  char *n = strdup(name.c_str());
  string fr;
  if ( !opt.skipFwd && !opt.skipRev ) fr = "both";
  if ( !opt.skipFwd &&  opt.skipRev ) fr = "fwd";
  if (  opt.skipFwd && !opt.skipRev ) fr = "rev";

  string rm;
  if ( opt.repeatMask ) rm="r"; else rm="nr";
					
    
  for(unsigned int i=0; i<strlen(n); i++)
    if(n[i]=='/' || n[i]=='.') n[i]='-';
    
  os << opt.cache_dir << "mc-" << n << "-";
  for(unsigned int p=0; p<pat->wordCount(); p++)
    os << hex << pat->readWord(p);
  os << "-" << fr;
  os << "-" << rm;
  os << "-" << dec << opt.rand_seed;
  os << "-" << hash_bits;
  free(n);
    
  return os.str();
}

RepeatMap::RepeatMap() : clusters() {}

void RepeatMap::add(const LocList &l){
  clusters.push_back(l);
}

void RepeatMap::writeOut(ostream &os){
  for(list<LocList>::iterator i=clusters.begin();i!=clusters.end();++i){
    LocList &l=*i;
    for(int s=0;s<seq_count;s++){
      if(s==0){
	assert(l[0].front().seqId()<seq_count);
	os << "R: "<<Window(l[0].front(),globalPat).prettyString()<<endl;
      }
      os << s << ": ";
      LocSubList::iterator lite=l[s].begin();
      do {
	SeqPos p=lite->seq()->extCoords(lite->pos);
	os << dstring(p);
	++lite;
	if(lite!=l[s].end())
	  os << " ";
	else{
	  os << endl;
	  break;
	}
      }while(1);
    }
    os << endl; //extra new line on group end.
  }
}

IntervalSet::IntervalSet() :
  spaces() {
  spaces.reserve(seq_count);
  if(opt.retainMembers)
    members[activeHash]++;
}

IntervalSet::IntervalSet(const AnchorSet& a):
  spaces() , members(a.members)
{
  spaces.reserve(seq_count);
  for(vector<MapIte>::const_iterator i=a.spaces.begin();
      i!=a.spaces.end();++i)
    spaces.push_back(i->key());
}

IntervalSet::IntervalSet(const _IntervalIterator &intStart, const _IntervalIterator &intStop,
			 const _MemberIterator &memberStart,const _MemberIterator &memberStop){
  spaces.reserve(seq_count);
  for(_IntervalIterator i=intStart;i!=intStop;++i)
    spaces.push_back(*i);
  for(_MemberIterator i=memberStart;i!=memberStop;++i)
    members.insert(*i);
}

ostream& operator<<(ostream& os,IntervalSet& a){
  os << "(";
  vector<UsedInt>::iterator i=a.spaces.begin();
  if(i==a.spaces.end())
    return os << ")";
  vector<UsedInt>::iterator next=i;
  int idx;
  for(next++,idx=0;next!=a.spaces.end();next++,i++,idx++)
    os << anchorCoords(*i,seqs[idx]) << ",";
  return os << anchorCoords(*i,seqs[idx]) << ")";
}

UsedInt anchorCoords(const UsedInt &b,Sequence *s){
  if(b.start<0)
    return s->length()-b;
  return b;
}

UsedInt operator-(SeqPos a,const UsedInt& b){
  return UsedInt(-1-(a+b.start),-1-(a+b.stop));
}
